import os
import pickle
import getpass
from . import config

def load(filename: str = config.save_file) -> dict:
    if os.path.exists(filename):
        with open(filename, 'rb') as f:
            return pickle.load(f)
    return {}

def save(data: dict, filename: str = config.save_file) -> None:
    with open(filename, 'wb') as f:
        pickle.dump(data, f)

class AutosavedPlayer:
    DEFAULT_ATTRIBUTES: dict[str, int, dict] = {
        'hp': 100,
        'level': 1,
        'experience': 0,
        'gold': 0,
        'last_claim_timestamp': None,
        'settings': {},
        'aliases': {},
    }

    def __init__(self) -> None:
        self.username = getpass.getuser()  # Automatically use the current system user
        self._data = self.load()
        self.automigrate()

        for key, value in self._data.items():
            setattr(self, key, value)

    def load(self, filename: str = config.save_file) -> dict:
        if os.path.exists(filename):
            data = pickle.load(open(filename, 'rb'))
            return data
        print("Welcome to idlegame, the pip & play Python game!")
        print("idlegame emulates a zsh command line to play. Get started: `man idlegame` / `man commands`")
        print("© 2024 Ben Boonstra MIT License.")
        return {}

    def save(self, filename: str = config.save_file) -> None:
        save(self._data)

    def __getattr__(self, attr: str) -> int:
        if attr in self._data:
            return self._data[attr]
        raise AttributeError(f"{attr} not found")

    def __setattr__(self, attr: str, value: int) -> None:
        if attr in ['username', '_data']:
            super().__setattr__(attr, value)
        else:
            self._data[attr] = value
            self.save()

    def automigrate(self) -> None:
        for attr, default_value in self.DEFAULT_ATTRIBUTES.items():
            if attr not in self._data:
                self._data[attr] = default_value

def handle_login() -> AutosavedPlayer:
    """Automatically login as the current system user."""
    username = getpass.getuser()
    print(f"Logged in as: {username}")

    return AutosavedPlayer()


from datetime import datetime, timezone
from .data import AutosavedPlayer

def handle_claim(player: AutosavedPlayer, *args, **kwargs) -> None:
    """Claim rewards in the game.

    Usage:
        claim [--reward <reward>] [--silent]
    
    Options:
        --reward <reward>  Specify the reward to claim.
        --silent           Suppress output messages.
    """
    now = datetime.now(timezone.utc)  # Use UTC time
    
    # Check for the --silent option
    silent_mode = kwargs.get('silent', False)

    if player.last_claim_timestamp is None:
        # If the player has never claimed, just set it to now
        player.last_claim_timestamp = now
        player.save()
        print("You have claimed your first reward of 100 gold!")
        player.gold += 100
        return

    # Calculate time difference since last claim
    time_offline = now - player.last_claim_timestamp
    minutes_offline = int(time_offline.total_seconds() // 60)
    gold_gain = minutes_offline
    if minutes_offline < 1:
        if not silent_mode:
            print("You haven't been offline for long enough to get any rewards!")
        return
    
    player.gold += gold_gain
    
    # Update last claim timestamp to now
    player.last_claim_timestamp = now
    
    if not silent_mode:
        print(f"You have claimed {gold_gain} gold for being offline for {minutes_offline} minutes!")
